package utopp.modell;

import java.io.*;
import java.text.*;

import utopp.xml.*;


public class UmlBinaerAssoziation  extends UmlAssoziation
{
	private String sTyp;
	private String sRichtung;
	private UmlAssRolle arRolle1;
	private UmlAssRolle arRolle2;
	
	public UmlBinaerAssoziation(String stereotyp, String typ, String name,  String richtung, 
	    String zusicherung, String eigenschaften, UmlAssRolle rolle1, UmlAssRolle rolle2)
	{
		super(stereotyp,name, zusicherung, eigenschaften);
		sTyp = typ;
		sRichtung = richtung;
		arRolle1 = rolle1;
		arRolle2 = rolle2;	
	}
	
	public UmlBinaerAssoziation(String stereotyp, String typ, String name,  String richtung, 
	    								String zusicherung, String eigenschaften)
	{
		this(stereotyp, typ, name, richtung, zusicherung, eigenschaften,null,null);
	}
	
	public UmlBinaerAssoziation(){
	  this("","","","ungerichtet","","",null,null);
	}
	
	public UmlBinaerAssoziation(Object obj){
	  this();
	  if (obj instanceof UmlMultiAssoziation)
	  {
	  	UmlMultiAssoziation ma = (UmlMultiAssoziation) obj;
		setStereotyp(ma.getStereotyp());
		setName(ma.getName());
		setZusicherung(ma.getZusicherung());
		setEigenschaften(ma.getEigenschaften()); 
		UmlRollenListe rl = ma.getRollen();
		if (rl.size()>0)
		{
			arRolle1 = (UmlAssRolle) rl.get(0);
			if (rl.size()>1)			  
			  arRolle2 = (UmlAssRolle) rl.get(1);			   
		}
		 
	  } // if
	}

    public String getRichtung(){
		return sRichtung;
	}
	
	public UmlAssRolle getRolle1 () 
	{
		return arRolle1;
	}

	public UmlAssRolle getRolle2 () 
	{
		return arRolle2;
	}
 
	public String getTyp () 
	{
		return sTyp;
		
	}
	
	public void setRichtung(String value){
		sRichtung = value;
	}

	public void setRolle1 (UmlKlasse klasse, String rollenName, String kardinalitaet) 
	{
		arRolle1 = new UmlAssRolle(klasse, rollenName, kardinalitaet);
	}
	
	public void setRolle1(UmlAssRolle value){
	  arRolle1 = value;
	}

	public void setRolle2 (UmlKlasse klasse, String rollenName, String kardinalitaet) 
	{
		arRolle2 = new UmlAssRolle(klasse, rollenName, kardinalitaet);
	}
	
	public void setRolle2(UmlAssRolle value){
	  arRolle2 = value;
	}

	public void setTyp (String value) 
	{
		sTyp = value;
	}
	
		
	public String toString(){
	  String typ = getTyp();
	  String name = getName();
	  return (typ==null || typ.equals(""))?name:typ+": "+name;
	}
	
	public void xmlWrite (XmlWriter o, int depth)  throws IOException
	{
	  o.writeTagLn("BinaerAssoziation", depth);
	  o.writeTaggedValueLn("Stereotyp",getStereotyp(),depth+1);	  
	  o.writeTaggedValueLn("Typ",getTyp(),depth+1);
	  o.writeTaggedValueLn("Name",getName(),depth+1);
	  o.writeTaggedValueLn("Richtung",getRichtung(),depth+1);
	  o.writeTaggedValueLn("Zusicherung",getZusicherung(),depth+1);
	  o.writeTaggedValueLn("Eigenschaften",getEigenschaften(),depth+1);
	  arRolle1.xmlWrite(o, depth+1);
	  arRolle2.xmlWrite(o, depth+1);
	  o.writeEndTagLn("BinaerAssoziation", depth);
	  
	};
	
	public void xmlRead (XmlReader i) throws IOException
	{
	  boolean first = true;
	  try{
	  while (i.ready())	
	  {
		ParseResult pr=i.getNext();
		int typ = pr.getState();
		String tag = pr.getTag();
		
		if (typ==3 && tag.equals("BinaerAssoziation"))
		  break;
		if (typ == 2)  
		{ 		
		    if (tag.equals("Stereotyp"))
		      setStereotyp(i.getTagValue(tag));
		    else		
		    if (tag.equals("Typ"))
		      sTyp = i.getTagValue(tag);
		    else    
		    if (tag.equals("Name"))
		      setName(i.getTagValue(tag));
		    else 
		    if (tag.equals("Richtung"))
		      sRichtung = i.getTagValue(tag);
			else   			 
		    if (tag.equals("Zusicherung"))
		      setZusicherung(i.getTagValue(tag));
		    else 
		    if (tag.equals("Eigenschaften"))
		      setEigenschaften(i.getTagValue(tag));
			else  
		    if (tag.equals("Rolle"))
			{   UmlAssRolle ur = new UmlAssRolle();
		        ur.xmlRead(i);	
				if (first)
				{
					first = false;
					arRolle1 = ur;
				}
				else
				  arRolle2 = ur;								   
			}		      
		    
		}  
		
	  } // while
	  
	  }
	  catch(ParseException pe){System.out.println(pe);};
	  
	};		
	

}

