package utopp.modell;

import java.util.*;
import java.io.*;
import java.text.*;

import java.awt.image.*;
import java.awt.*;
import java.awt.geom.*;

import utopp.modell.event.*;
import utopp.xml.*;
import utopp.diagram.*;

public class UmlKlasse extends UmlObject implements Viewable
{
	private String sStereotyp;
	private String sSichtbarkeit;
	private String sPaket;
	private String sZusicherung;
	private String sEigenschaften;
	
	private UmlMethodenListe mlMethoden;
	private UmlAttributeListe alAttribute;
	
        private int thisWidth=200;
        private int thisHeight=400;
        private int lineDiff=15;
	
	public UmlKlasse(String name, String stereotyp, String sichtbarkeit, String paket,
		String zusicherung, String eigenschaften)
	{
		super(name);
		sStereotyp = stereotyp;
		sSichtbarkeit = sichtbarkeit;
		sPaket = paket;
		sZusicherung = zusicherung;
	  	sEigenschaften = eigenschaften;
	  	mlMethoden = new UmlMethodenListe();
	  	alAttribute = new UmlAttributeListe();
	}	
	
	public UmlKlasse()
	{
		this("","","","","","");
	}
	 
	public String getStereotyp () 
	{
		return sStereotyp;
	}
	
	public String getSichtbarkeit () 
	{
		return sSichtbarkeit;
	}

	public String getPaket () 
	{
		return sPaket;
	}

	public String getZusicherung () 
	{
		return sZusicherung;
	}

	public String getEigenschaften () 
	{
		return sEigenschaften;
	}

	public void setStereotyp (String value) 
	{
		sStereotyp = value;
	}

	public void setSichtbarkeit (String value) 
	{
		sSichtbarkeit = value;
	}

	public void setPaket (String value) 
	{
		sPaket = value;
	}

	public void setZusicherung (String value) 
	{
		sZusicherung = value;
	}

	public void setEigenschaften (String value) 
	{
		sEigenschaften = value;
	}
	
	public UmlMethodenListe getMethoden()
	{
	  return mlMethoden;
	}
	
	public UmlAttributeListe getAttribute()
	{
	  return alAttribute;	
	}
	
	public void setMethoden(UmlMethodenListe meth) throws ElementExistiertException
	{ 
	  mlMethoden = meth;
	}
	
	public void setAttribute(UmlAttributeListe attrib) throws ElementExistiertException
	{
	  alAttribute = attrib;
	}
	
	public String toString(){
	  return (sPaket==null || sPaket.equals(""))?getName():sPaket+"."+getName();
	}
			
	public boolean equals(Object obj){
	  return (obj!=null)?obj.toString().equals(toString()):false;
	}
	
	public void xmlWrite (XmlWriter o, int depth)  throws IOException
	{	  	
	  o.writeTagLn("Klasse", depth);
	  o.writeTaggedValueLn("Stereotyp",sStereotyp,depth+1);
	  o.writeTaggedValueLn("Sichtbarkeit",sSichtbarkeit,depth+1);
	  o.writeTaggedValueLn("Paket",sPaket,depth+1);
	  o.writeTaggedValueLn("Name",getName(),depth+1);
	  o.writeTaggedValueLn("Zusicherung",sZusicherung,depth+1);
	  o.writeTaggedValueLn("Eigenschaften",sEigenschaften,depth+1);
	  alAttribute.xmlWrite(o,depth+1);
	  mlMethoden.xmlWrite(o,depth+1);	  
	  o.writeEndTagLn("Klasse",depth);
	  
	};
	
	public void xmlRead (XmlReader i) throws IOException
	{	  
	  try{
	  while (i.ready())	
	  {
		ParseResult pr=i.getNext();
		int typ = pr.getState();
		String tag = pr.getTag();
		
		if (typ==3 && tag.equals("Klasse"))
		  break;
		if (typ == 2)  
		{
		    if (tag.equals("Stereotyp"))
		      sStereotyp = i.getTagValue(tag);
		    else  
		    if (tag.equals("Sichtbarkeit"))
		      sSichtbarkeit = i.getTagValue(tag);
		    else 		
		    if (tag.equals("Paket"))
		      sPaket = i.getTagValue(tag);
		    else  
		    if (tag.equals("Name"))
		      setName(i.getTagValue(tag));
			else 
		    if (tag.equals("Zusicherung"))
		      sZusicherung = i.getTagValue(tag);
		    else  
		    if (tag.equals("Eigenschaften"))
		      sEigenschaften = i.getTagValue(tag);
		    else 
		    if (tag.equals("Attributliste"))
			 alAttribute.xmlRead(i);
			else
			if (tag.equals("Methodenliste"))
			 mlMethoden.xmlRead(i); 
		}  
		
	  } // while
	  
	  }
	  catch(ParseException pe){System.out.println(pe);};
		//	
	};			

  public int determineImageWidth()
    {
    int maxLetters=0;//Breite
    if(sStereotyp.length()>maxLetters)
      maxLetters=sStereotyp.length();
    if((sPaket+getName()).length()>maxLetters)
      maxLetters= (sPaket+getName()).length();
    if(sZusicherung.length()>maxLetters)
      maxLetters= sZusicherung.length();
    Iterator it1 = alAttribute.iterator();
    while(it1.hasNext())
      {
      int buffLength = it1.next().toString().length();
      if(buffLength>maxLetters)
        maxLetters=buffLength;
      }
    Iterator it2 = mlMethoden.iterator();
    while(it2.hasNext())
      {
      int buffLength = it2.next().toString().length();
      if(buffLength>maxLetters)
        maxLetters=buffLength;
      }
    return maxLetters*7;
    }//end determineImageWidth

  public int determineImageHeight()
    {
    //HHE
    int maxListHeight=0;
    maxListHeight=alAttribute.size()+mlMethoden.size();
    maxListHeight+=6;
    return maxListHeight*lineDiff;
    }//end determineImageHeight
       

  public BufferedImage getImage()
    {
    //TEST
    thisWidth=determineImageWidth();
    thisHeight=determineImageHeight();
    BufferedImage buff = new BufferedImage(thisWidth,thisHeight,BufferedImage.TYPE_INT_ARGB);
    createPicture(buff.createGraphics());
    return buff;
    }

  public DiagramObject getDiagramObject()
    {
    //TEST
    return new DiagramObject(getImage(),this.getName(),"UmlKlasse");
    }

  public void createPicture(Graphics2D g)
    {
    //TEST
    Font f = new Font("SansSerif", Font.BOLD, 12);
    FontMetrics fm = g.getFontMetrics(f);
    int stringHeight = fm.getHeight();
    int stringYPos = 0;
    g.setFont(f);
    //Deklaration der Rechtecke und weisser Hintergrund.
    g.setColor(Color.white);
    Rectangle2D.Double rect3 = new Rectangle2D.Double(0,0,thisWidth-1,thisHeight-1);
    g.fill(rect3);
    //Rechtecke
    g.setColor(Color.black);
    g.draw(rect3);
    //Stereotyp
    stringYPos+=lineDiff;
    int stringWidth = fm.stringWidth("<<"+sStereotyp+">>");
    int stringXPos = (thisWidth - stringWidth)/2;
    g.drawString("<<"+sStereotyp+">>",stringXPos,stringYPos);
    //Paket+Name
    stringYPos+=lineDiff;
    stringWidth = fm.stringWidth(sPaket+"::"+getName());
    stringXPos = (thisWidth - stringWidth)/2;
    g.setColor(Color.red);
    g.drawString(sPaket+"::"+getName(),stringXPos,stringYPos);
    g.setColor(Color.black);
    //Zusicherung
    stringYPos+=lineDiff;
    stringWidth = fm.stringWidth("{"+sZusicherung+"}");
    stringXPos = (thisWidth - stringWidth)/2;
    g.drawString("{"+this.sZusicherung+"}",stringXPos,stringYPos);
    //erste Trennlinie
    stringYPos+=lineDiff;
    g.drawLine(0,stringYPos,thisWidth,stringYPos);
   
    //Alle Attribute
    Iterator it1 = alAttribute.iterator();
    while(it1.hasNext())
      {
      String buffString = it1.next().toString();
      stringYPos+=lineDiff;
      stringWidth = fm.stringWidth(buffString);
      stringXPos = (thisWidth - stringWidth)/2;
      g.drawString(buffString,stringXPos,stringYPos);
      }

    //zweite Trennlinie
    stringYPos+=lineDiff;
    g.drawLine(0,stringYPos,thisWidth,stringYPos);
        

    //Alle Methoden
    Iterator it2 = mlMethoden.iterator();
    while(it2.hasNext())
      {
      String buffString = it2.next().toString();
      stringYPos+=lineDiff;
      stringWidth = fm.stringWidth(buffString);
      stringXPos = (thisWidth - stringWidth)/2;
      System.out.println(buffString);
      g.drawString(buffString,stringXPos,stringYPos);
      }
       
 
    }        

}